<?php

/**
 * @file
 * Contains a Logger
 *
 * @license GPL v2 http://www.fsf.org/licensing/licenses/gpl.html
 * @author Chris Skene chris at previousnext dot com dot au
 * @copyright Copyright(c) 2014 Previous Next Pty Ltd
 */

namespace Drupal\ghost\Logger;

/**
 * Class Logger
 * @package Drupal\ghost\Utilities
 */
class Logger implements LoggerInterface {

  /**
   * The module logging.
   *
   * @var string
   */
  protected $caller;

  /**
   * The message to log.
   *
   * @var string
   */
  protected $message;

  /**
   * Variables to substitute into the message.
   *
   * @var array
   */
  protected $vars = array();

  /**
   * Whether to also send to the message area.
   *
   * @var bool
   */
  protected $useMessageArea = FALSE;

  /**
   * The severity
   *
   * @var int
   */
  protected $severity;

  /**
   * Constructor.
   *
   * @param string $module
   *   The module logging.
   */
  public function __construct($module) {

    $this->caller = $module;
  }

  /**
   * Constructor.
   *
   * @param string $caller
   *   The module calling the Log function.
   * @param string $message
   *   The message to log.
   * @param array $vars
   *   (Optional) Variables to substitute into the message.
   * @param int $severity
   *   (Optional) A Watchdog severity level. Defaults to WATCHDOG_NOTICE.
   * @param bool $message_area
   *   (Optional) Whether to log to drupal_set_message as well. Defaults to
   *   FALSE.
   *
   * @return Logger
   *   This Logger
   */
  static public function init($caller, $message = NULL, $vars = array(), $severity = WATCHDOG_NOTICE, $message_area = FALSE) {

    $logger = new static($caller);
    $logger->setMessage($message);
    $logger->setVars($vars);
    $logger->setSeverity($severity);
    $logger->useMessageArea = $message_area;

    return $logger;
  }

  /**
   * Log the message.
   */
  public function log() {

    $this->logWrite();
    $this->logMessage();
  }

  /**
   * Log the message using drupal_set_message().
   */
  public function logMessage() {
    if ($this->isUseMessageArea() == TRUE) {

      $this->screenWrite();
    }
    else {
      $this->logWrite();
    }
  }

  /**
   * Getter for module.
   *
   * @return string
   *   The module name.
   */
  public function getCaller() {

    return check_plain($this->caller);
  }

  /**
   * Getter for message.
   *
   * @return string
   *   The message.
   */
  public function getMessage() {

    return $this->message;
  }

  /**
   * Setter for message.
   *
   * @param string $message
   *   The value for message.
   *
   * @return Logger
   *   An instance of Logger
   */
  public function setMessage($message) {

    $this->message = $message;

    return $this;
  }

  /**
   * Getter for severity.
   *
   * @return int
   *   The severity.
   */
  public function getSeverity() {

    if (isset($this->severity)) {
      return $this->severity;
    }

    return WATCHDOG_NOTICE;
  }

  /**
   * Setter for severity.
   *
   * @param int $severity
   *   (optional) The value for severity. Defaults to WATCHDOG_NOTICE.
   *
   * @return Logger
   *   An instance of Logger
   */
  public function setSeverity($severity  = WATCHDOG_NOTICE) {

    $this->severity = $severity;

    return $this;
  }

  /**
   * Getter for vars.
   *
   * @return array
   *   The vars.
   */
  public function getVars() {

    if (isset($this->vars)) {
      return $this->vars;
    }

    return array();
  }

  /**
   * Setter for vars.
   *
   * @param array $vars
   *   The value for vars.
   *
   * @return Logger
   *   An instance of Logger
   */
  public function setVars($vars) {

    $this->vars = $vars;

    return $this;
  }

  /**
   * Getter for vars.
   *
   * @param string $key
   *   Key of the var to return.
   *
   * @return array
   *   The vars.
   */
  public function getVar($key) {

    if (isset($this->vars[$key])) {
      return $this->vars[$key];
    }

    return NULL;
  }

  /**
   * Setter for vars.
   *
   * @param string $key
   *   The key to set.
   * @param mixed $value
   *   The value for vars.
   *
   * @return Logger
   *   An instance of Logger
   */
  public function setVar($key, $value) {

    $this->vars[$key] = $value;

    return $this;
  }

  /**
   * Should this be sent to the message area.
   *
   * @return bool
   *   TRUE if it should
   */
  public function isUseMessageArea() {

    if (isset($this->useMessageArea)) {
      return $this->useMessageArea;
    }

    return FALSE;
  }

  /**
   * Whether to write this message to the screen.
   *
   * @param bool $state
   *   TRUE if using drupal_set_message().
   *
   * @return Logger
   *   This logger.
   */
  public function useMessageArea($state = TRUE) {

    $this->useMessageArea = $state;

    return $this;
  }

  /**
   * Write a message to watchdog.
   *
   * This is part of the LogInterface and should be implemented by Logger
   * implementations, but should not be called by users.
   *
   * @see log()
   */
  public function logWrite() {

    watchdog($this->getCaller(), $this->getMessage(), $this->getVars(), $this->getSeverity());
  }

  /**
   * Log a message to the screen (drupal_set_message).
   */
  public function screenWrite() {

    $dsm_severity = 'status';
    if (in_array($this->getSeverity(), array(
      WATCHDOG_ERROR,
      WATCHDOG_ALERT,
      WATCHDOG_CRITICAL,
      WATCHDOG_EMERGENCY,
    ))) {
      $dsm_severity = 'error';
    }
    elseif (in_array($this->getSeverity(), array(WATCHDOG_WARNING))) {
      $dsm_severity = 'warning';
    }

    drupal_set_message(format_string($this->getMessage(), $this->getVars()), $dsm_severity, FALSE);
  }
}
